;=========================================================================
.model small, c
.386
include support.asm
.stack
;=========================================================================
.data
;=========================================================================

;=========================================================================
.code
;=========================================================================

;----------------------------------------------------------
; This procedure checks for a mouse driver, resets it, and
; installs the interrupt subroutine. Returns non-zero on
; success or zero on failure.
;----------------------------------------------------------

MouseInit proc

    ;-------------------------------------------------------
    ; Use the DOS Get Interrupt Vector function to get the
    ; interrupt 33h vector. If the segment address is zero,
    ; then the mouse driver is not installed.
    ;-------------------------------------------------------

    mov ax, 3533h
    int 21h
    mov ax, es
    test ax, ax
    jnz @F
    ret
  @@:

    ;-------------------------------------------------
    ; Attempt to reset the mouse driver. If the reset
    ; fails, then the mouse driver is not installed.
    ;-------------------------------------------------

    xor ax,ax
    int 33h
    test ax, ax
    jnz @F
    ret
  @@:

    ;---------------------------------------------------------------
    ; Install the interrupt subroutine, specifying a condition mask
    ; that will cause the mouse driver to call the subroutine for
    ; any of the possible conditions.
    ;
    ; The function expects ES to contain the segment address of
    ; the interrupt subroutine.
    ;---------------------------------------------------------------

    mov ax, 12                    ; function number
    mov cx, 11111b                ; condition mask
    push cs
    pop es
    mov dx, InterruptSub
    int 33h

    mov ax, 1
    ret

MouseInit endp

;=========================================================================

;-----------------------------------------------------------------
; This procedure is the interrupt subroutine, called by the mouse
; driver whenever any of the conditions specifed in the condition
; mask change.
;
; Note the far return at the end, required here because the mouse
; driver calls this proc via a FAR CALL.
;
; Since we cannot depend on DS containing a known value when this
; procedure is called, to access data in our data segment without
; a segment override (or use the procedures in support.asm) we
; need to set DS to the segment address of our data segment.
; Because this program uses the .startup directive, we can safely
; assume that SS contains the segment address of our data segment.
;
; For the examples in the Microsoft Mouse Programmer's Reference,
; 1991, the interrupt subroutines do not attempt to preserve any
; registers. Even though under Windows 2000 this procedure will
; work without preserving any of the registers it changes, for
; compatibility with other mouse drivers I chose to preserve the
; DS segment register.
;
; The mouse cursor position is reported in mouse driver virtual-
; screen coordinates, in this case for a 640x200 virtual screen.
;-----------------------------------------------------------------

InterruptSub proc

    push ds                     ; preserve DS before we change it
    push ss
    pop ds                      ; set DS to our data segment
    loc 10,0
    print binword$(ax),13,10    ; display condition mask
    print binword$(bx),13,10    ; display button state
    print word$(cx),"   ",13,10 ; display cursor x coordinate
    print word$(dx),"   "       ; display cursor y coorddnate
    pop ds                      ; recover original DS.

    retf

InterruptSub endp

;=========================================================================
.startup
;=========================================================================
    mov ax, 13h
    int 10h

    call MouseInit
    print word$(ax),13,10

    mov ax, 1                   ; show cursor
    int 33h

    call waitkey
;=========================================================================
.exit
end

